package sun.java2d;

import java.awt.peer.ComponentPeer;
import java.awt.Insets;
import java.awt.GraphicsConfiguration;
import java.awt.Toolkit;
import java.awt.Rectangle;
import java.util.TreeMap;
import java.util.Locale;
import java.util.Arrays;
import sun.font.FontManagerFactory;
import sun.font.FontManagerForSGE;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.AWTError;
import java.security.AccessController;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.io.File;
import java.security.PrivilegedAction;
import sun.awt.SunDisplayChanger;
import java.awt.GraphicsDevice;
import java.awt.Font;
import sun.awt.DisplayChangedListener;
import java.awt.GraphicsEnvironment;

public abstract class SunGraphicsEnvironment extends GraphicsEnvironment implements DisplayChangedListener
{
    public static boolean isOpenSolaris;
    private static Font defaultFont;
    protected GraphicsDevice[] screens;
    protected SunDisplayChanger displayChanger;
    
    public SunGraphicsEnvironment() {
        this.displayChanger = new SunDisplayChanger();
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                String property = System.getProperty("os.version", "0.0");
                try {
                    if (Float.parseFloat(property) > 5.1f) {
                        FileInputStream in = new FileInputStream(new File("/etc/release"));
                        if (new BufferedReader(new InputStreamReader(in, "ISO-8859-1")).readLine().indexOf("OpenSolaris") >= 0) {
                            SunGraphicsEnvironment.isOpenSolaris = true;
                        }
                        else {
                            SunGraphicsEnvironment.isOpenSolaris = !new File("/usr/openwin/lib/X11/fonts/TrueType/CourierNew.ttf").exists();
                        }
                        in.close();
                    }
                }
                catch (Exception ex) {}
                SunGraphicsEnvironment.defaultFont = new Font("Dialog", 0, 12);
                return null;
            }
        });
    }
    
    @Override
    public synchronized GraphicsDevice[] getScreenDevices() {
        GraphicsDevice[] screens = this.screens;
        if (screens == null) {
            int numScreens = this.getNumScreens();
            screens = new GraphicsDevice[numScreens];
            for (int i = 0; i < numScreens; ++i) {
                screens[i] = this.makeScreenDevice(i);
            }
            this.screens = screens;
            return screens;
        }
        return screens;
    }
    
    protected abstract int getNumScreens();
    
    protected abstract GraphicsDevice makeScreenDevice(int p0);
    
    @Override
    public GraphicsDevice getDefaultScreenDevice() {
        GraphicsDevice[] screenDevices = this.getScreenDevices();
        if (screenDevices.length == 0) {
            throw new AWTError("no screen devices");
        }
        return screenDevices[0];
    }
    
    @Override
    public Graphics2D createGraphics(BufferedImage bufferedImage) {
        if (bufferedImage == null) {
            throw new NullPointerException("BufferedImage cannot be null");
        }
        return new SunGraphics2D(SurfaceData.getPrimarySurfaceData(bufferedImage), Color.white, Color.black, SunGraphicsEnvironment.defaultFont);
    }
    
    public static FontManagerForSGE getFontManagerForSGE() {
        return (FontManagerForSGE)FontManagerFactory.getInstance();
    }
    
    public static void useAlternateFontforJALocales() {
        getFontManagerForSGE().useAlternateFontforJALocales();
    }
    
    @Override
    public Font[] getAllFonts() {
        FontManagerForSGE fontManagerForSGE = getFontManagerForSGE();
        Font[] allInstalledFonts = fontManagerForSGE.getAllInstalledFonts();
        Font[] createdFonts = fontManagerForSGE.getCreatedFonts();
        if (createdFonts == null || createdFonts.length == 0) {
            return allInstalledFonts;
        }
        Font[] array = (Font[])Arrays.copyOf(allInstalledFonts, allInstalledFonts.length + createdFonts.length);
        System.arraycopy(createdFonts, 0, array, allInstalledFonts.length, createdFonts.length);
        return array;
    }
    
    @Override
    public String[] getAvailableFontFamilyNames(Locale locale) {
        FontManagerForSGE fontManagerForSGE = getFontManagerForSGE();
        String[] installedFontFamilyNames = fontManagerForSGE.getInstalledFontFamilyNames(locale);
        TreeMap<String, String> createdFontFamilyNames = fontManagerForSGE.getCreatedFontFamilyNames();
        if (createdFontFamilyNames == null || createdFontFamilyNames.size() == 0) {
            return installedFontFamilyNames;
        }
        for (int i = 0; i < installedFontFamilyNames.length; ++i) {
            createdFontFamilyNames.put(installedFontFamilyNames[i].toLowerCase(locale), installedFontFamilyNames[i]);
        }
        String[] array = new String[createdFontFamilyNames.size()];
        Object[] array2 = createdFontFamilyNames.keySet().toArray();
        for (int j = 0; j < array2.length; ++j) {
            array[j] = (String)createdFontFamilyNames.get(array2[j]);
        }
        return array;
    }
    
    @Override
    public String[] getAvailableFontFamilyNames() {
        return this.getAvailableFontFamilyNames(Locale.getDefault());
    }
    
    public static Rectangle getUsableBounds(GraphicsDevice graphicsDevice) {
        GraphicsConfiguration defaultConfiguration = graphicsDevice.getDefaultConfiguration();
        Insets screenInsets = Toolkit.getDefaultToolkit().getScreenInsets(defaultConfiguration);
        Rectangle bounds = defaultConfiguration.getBounds();
        bounds.x += screenInsets.left;
        bounds.y += screenInsets.top;
        bounds.width -= screenInsets.left + screenInsets.right;
        bounds.height -= screenInsets.top + screenInsets.bottom;
        return bounds;
    }
    
    @Override
    public void displayChanged() {
        for (GraphicsDevice graphicsDevice : this.getScreenDevices()) {
            if ((graphicsDevice instanceof DisplayChangedListener)) {
                ((DisplayChangedListener)graphicsDevice).displayChanged();
            }
        }
        this.displayChanger.notifyListeners();
    }
    
    @Override
    public void paletteChanged() {
        this.displayChanger.notifyPaletteChanged();
    }
    
    public abstract boolean isDisplayLocal();
    
    public void addDisplayChangedListener(DisplayChangedListener theListener) {
        this.displayChanger.add(theListener);
    }
    
    public void removeDisplayChangedListener(DisplayChangedListener theListener) {
        this.displayChanger.remove(theListener);
    }
    
    public boolean isFlipStrategyPreferred(ComponentPeer componentPeer) {
        return false;
    }
}
